"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const os_1 = require("os");
class WebpackCommandPlugin {
    constructor(options) {
        this.options = {
            onBuildStart: [],
            onBuildEnd: [],
            onBuildExit: [],
            dev: true,
            safe: false
        };
        this.validateInput(options);
    }
    /**
     * 处理传入参数
     * @param options
     */
    validateInput(options) {
        if (options.onBuildStart) {
            this.options.onBuildStart = options.onBuildStart.split('&&');
        }
        if (options.onBuildEnd) {
            this.options.onBuildEnd = options.onBuildEnd.split('&&');
        }
        if (options.onBuildExit) {
            this.options.onBuildExit = options.onBuildExit.split('&&');
        }
        if (options.dev) {
            this.options.dev = options.dev;
        }
        if (options.safe) {
            this.options.safe = options.safe;
        }
    }
    /**
     * webpack 钩子函数
     * @param compiler
     */
    apply(compiler) {
        // 编译(compilation)创建之后
        compiler.hooks.compilation.tap('WebpackShellPlugin', () => {
            if (this.options.onBuildStart && this.options.onBuildStart.length) {
                this.runCommands(this.options.onBuildStart);
                if (this.options.dev) {
                    this.options.onBuildStart = [];
                }
            }
        });
        // 生成资源到 output 目录之后
        compiler.hooks.afterEmit.tapAsync('WebpackShellPlugin', (_compilation, callback) => {
            if (this.options.onBuildEnd && this.options.onBuildEnd.length) {
                this.runCommands(this.options.onBuildEnd);
                if (this.options.dev) {
                    this.options.onBuildEnd = [];
                }
            }
            callback();
        });
        // 编译(compilation)完成。
        compiler.hooks.done.tap('WebpackShellPlugin', () => {
            if (this.options.onBuildExit && this.options.onBuildExit.length) {
                this.runCommands(this.options.onBuildExit);
            }
        });
    }
    /**
     * 执行shell
     * @param commands
     */
    runCommands(commands) {
        for (const item of commands) {
            if (os_1.platform() === 'win32' || this.options.safe) {
                this.spreadStdoutAndStdErr(child_process_1.exec(item, this.throwError));
            }
            else {
                const { command, args } = this.serializeScript(item);
                const proc = child_process_1.spawn(command, args, { stdio: 'inherit' });
                proc.on('close', this.throwError);
            }
        }
    }
    /**
     * 处理spawn方法参数
     * @param script
     */
    serializeScript(script) {
        const [command, ...args] = script.split(' ');
        return { command, args };
    }
    /**
     * 向主进程输出结果
     * @param proc
     */
    spreadStdoutAndStdErr(proc) {
        if (proc && proc.stdout && proc.stderr) {
            proc.stdout.pipe(process.stdout);
            proc.stderr.pipe(process.stdout);
        }
    }
    /**
     * 错误处理
     * @param error
     */
    throwError(error) {
        if (error) {
            throw error;
        }
    }
}
exports.default = WebpackCommandPlugin;
//# sourceMappingURL=index.js.map